<?php

namespace Drupal\donorperfect_donor\Entity;

use Drupal\Component\Utility\NestedArray;
use Drupal\donorperfect\DPQuery;
use Drupal\donorperfect\DPUtility;
use Drupal\donorperfect\Entity\ControllerBase;

/**
 * Controller class for DonorPerfect Donor entities.
 */
class DonorController extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  const ENTITY_TYPE_ID = 'donorperfect_donor';

  /**
   * {@inheritdoc}
   */
  public function getSettingsFormClass() {
    return '\Drupal\donorperfect_donor\Form\SettingsForm';
  }

  /**
   * Search for DonorPerfect donors that match given conditions.
   *
   * @param array $conditions
   * @param string $join_type
   *
   * @return array
   *   Array of donor_ids that match given conditions.
   */
  public static function search(array $conditions=[], string $join_type='left') {
    $results = [];
    if (!empty($conditions)) {
      $dptables = DPUtility::getCache('tables');
      $query = new DPQuery('select', 'dp.donor_id');
      $query->setJoinType($join_type);
      foreach ($conditions as $key => $info) {
        $info['value'] = trim($info['value'] ?? '');
        // Do alpha_dash validation on the value to protect against SQL injection.
        if (DPUtility::validateAlphaDash($info['value'])) {
          if (($info['table'] == 'dp') && ($info['field'] == 'first_name')) {
            // Test for different variations on the first name.
            $variations = \Drupal::config('donorperfect.settings')->get('first_name_variations') ?? [];
            $condition = '';
            $temp_conditions = [];
            $value_lower = strtolower($info['value']);
            foreach ($variations as $group) {
              if (in_array($value_lower, $group)) {
                foreach ($group as $variation) {
                  $temp_conditions[] = "(dp.first_name LIKE '%" . str_replace("'", "''", $variation) . "%')";
                }
              }
              if (!empty($temp_conditions)) {
                $condition = '(' . implode(' OR ', $temp_conditions) . ')';
              }
            }
            if (empty($condition)) {
              $condition = "dp.first_name LIKE '%" . str_replace("'", "''", $info['value']) . "%'";
            }
            $query->addCondition($condition);
          } elseif (!empty($info['table']) && !empty($info['field']) && ($field_info = NestedArray::getValue($dptables, [$info['table'], $info['field']]))) {
            $wildcard = FALSE;
            switch ($field_info['mysql_data_type']) {
              case 'varchar':
              case 'nvarchar':
              case 'char':
              case 'text':
                $operator = ' LIKE ';
                if (isset($info['wildcard'])) $wildcard = $info['wildcard'];
                break;
              default:
                $operator = '=';
                break;
            }
            if (!empty($wildcard)) {
              switch ($wildcard) {
                case 'before':
                  $info['value'] = '%' . $info['value'];
                  break;
                case 'after':
                  $info['value'] = $info['value'] . '%';
                  break;
                case 'both':
                  $info['value'] = '%' . $info['value'] . '%';
                  break;
              }
            }
            $value = $field_info['quote'] ? "'" . str_replace("'", "''", $info['value']) . "'" : $info['value'];
            if (isset($info['operator'])) $operator = $info['operator'];
            $condition = $info['table'] . '.' . $info['field'] . $operator . $value;
            $query->addCondition($condition);
            // Make sure the table is included in the query.
            if ($info['table'] !== 'dp') {
              $matches = [];
              if (preg_match('/^dp(.*)udf$/', $info['table'], $matches)) {
                $query->addField($info['table'] . '.' . $matches[1] . '_id');
              } else {
                $query->addField($info['table'] . '.donor_id');
              }
            }
          } else {
            unset($conditions[$key]);
          }
        } else {
          unset($conditions[$key]);
        }
      }
      if (!empty($conditions)) {
        // Set ORDER BY.
        $query->addOrder('dp.last_name');
        $query->addOrder('dp.first_name');
        // Execute the query.
        $query->execute();
        // Reformat results into an array of donor_ids.
        if (!empty($query->result) && is_array($query->result)) {
          foreach ($query->result as $result) {
            if (is_array($result) && !empty($result['donor_id'])) {
              $results[$result['donor_id']] = (int)$result['donor_id'];
            }
          }
        }
      }
    }
    return $results;
  }

}
